@@ -15,11 +15,8 @@ firm.eff <- rnorm(nlevels(firm))
 ## left hand side
 y <- x + 0.25*x2 + id.eff[id] + firm.eff[firm] + rnorm(length(x))
 
-## make a data frame
-dset <- data.frame(y,x,x2,id,firm)
-
 ## estimate and print result
-est <- felm(y ~ x+x2,fl=list(id=id,firm=firm),data=dset)
+est <- felm(y ~ x+x2+G(id)+G(firm))
 summary(est)
 
 ## extract the group fixed effects
